/*************************************************************************\
 * DV-Load by Binary Nine (Nicholas J. Leon, author)                     *
 * A performance monitor for DESQView                                    *
 *************************************************************************
 * Inspired by DVMON by Barry A. Burke.                                  *
 * Written under Borland C++ 2.0. But with a little modification can be  *
 * compiled under any compiler.                                          *
 *                                                                       *
 *    Copyright (c) 1992 Binary Nine. All rights, except those           *
 *    specifically granted herein are reserved by the author. The right  *
 *    to copy and distribute this material is granted without fee for    *
 *    any and all non-commercial use. This material specifically may     *
 *    not be distributed or sold for a fee nor incorporated in whole or  *
 *    in part into any other product that is distributed or sold for a   *
 *    fee without specific permission of the author. To obtain special   *
 *    permission or to report any difficulties with this material        *
 *    contact:                                                           *
 *                      Binary Nine                                      *
 *                      c/o Nicholas J. Leon                             *
 *                      Route 2, Box 422                                 *
 *                      Randleman, NC  27317                             *
 *                                                                       *
 *    THIS MATERIAL IS DISTRIBUTED "as is" WITHOUT ANY EXPRESSED OR      *
 *    IMPLIED WARRANTY OR LIABILITY FOR DIRECT, INDIRECT OR              *
 *    CONSEQUENTIAL DAMAGES.                                             *
\*************************************************************************/
 
#include <time.h>
#include <dos.h>
#include <stdlib.h>
#include <stdio.h>

#define SAMPLE       300000     // size of measurement loop
#define GRAPHLENGTH  20         // size of the bar graph

// some typedef's I almost always use (me? A past assembly programmer?
// Nah!! ;-) )

typedef unsigned char byte;
typedef unsigned int  word;
typedef unsigned long dword;

/*************************************************************************\
 * I inlined these next two functions for speed purposes. It seemed kida *
 * silly to invoke the DV "PAUSE" and have the overhead for the call.    *
 * Normally I'd use asm {} to generate the interrupt, but you can't      *
 * inline asm {}.                                                        *
 * I decided to inline DVpositionCursor because I call it so often.      *
\*************************************************************************/

// give up remainder of time slice

inline void DVgiveupCPU(void)
{
    _AX=0x1000;
    geninterrupt(0x15);
}

// standard BIOS function for cursor positioning

inline void DVpositionCursor(byte x,byte y)
{
    _BH=0;
    _AH=2;
    _DH=x;
    _DL=y;
    geninterrupt(0x10);
}

// returns a non-zero if DV is running

word DVrunning(void)
{
    asm {
        mov ax,0x1022
        mov bx,0
        int 0x15
        }

    return (_BX);
}

// tells DV not to time slice

void DVbeginCritical(void)
{
    asm {
        mov ax,0x101b
        int 0x15
        }
}

// tells DV to time slice again

void DVendCritical(void)
{
    asm {
        mov ax,0x101c
        int 0x15
        }
}

// counts the number of ticks it takes to execute a very tight
// loop

dword numberTicks(dword lCount)
{
    clock_t cStart;
    clock_t cEnd;

    cStart=clock();

    for (;lCount;lCount--);

    cEnd=clock();

    return (dword)(cEnd-cStart);
}

// draws the ticks on the bar graph

void drawTicks(void)
{
    DVpositionCursor(2,0);

    printf("");
}

// draws the bar graph

void drawBar(byte bLength)
{
    byte bBar[GRAPHLENGTH+1];
    byte bCount;

    DVpositionCursor(1,0);

    for (bCount=0;bCount<=bLength;bCount++)
        bBar[bCount]='';

    for (bCount=bLength;bCount<GRAPHLENGTH;bCount++)
        bBar[bCount]=' ';

    bBar[GRAPHLENGTH]=NULL;

    printf(bBar);
}

// the main function

void main(void)
{
    dword   dwCalibration;
    dword   dwAmount;
    float   fLoad;
    byte    bBarLength;
    time_t  tCurrentTime;

    if (!DVrunning())
    {
        printf("This is a DESQView only program.\n");
        return;
    }

    drawTicks();

    DVpositionCursor(1,0);

    printf("Calibrating...");

    DVbeginCritical();

    dwCalibration=numberTicks(SAMPLE);

    DVendCritical();

    DVpositionCursor(1,0);

    printf("              ");

    DVgiveupCPU();

    for(;;)
    {
        dwAmount=numberTicks(SAMPLE);

        fLoad=(float)dwCalibration/dwAmount;
        fLoad=1-fLoad;

        if (fLoad<0)
           fLoad=0;

        bBarLength=fLoad*GRAPHLENGTH;

        drawBar(bBarLength);

        DVpositionCursor(3,0);

        printf("CPU Load: %d%%  ",(int)(fLoad*100));

        time(&tCurrentTime);

        DVpositionCursor(0,0);

        printf(ctime(&tCurrentTime));

        DVgiveupCPU();
    }

}

